<?php
/**
 * @package   Essentials YOOtheme Pro 2.2.10 build 0508.0933
 * @author    ZOOlanders https://www.zoolanders.com
 * @copyright Copyright (C) Joolanders, SL
 * @license   http://www.gnu.org/licenses/gpl.html GNU/GPL
 */

namespace ZOOlanders\YOOessentials\Source\Provider\GoogleCalendar\Type;

use ZOOlanders\YOOessentials\Source\GraphQL\AbstractQueryType;
use ZOOlanders\YOOessentials\Source\GraphQL\HasSourceInterface;
use ZOOlanders\YOOessentials\Source\Resolver\CachesResolvedSourceData;
use ZOOlanders\YOOessentials\Source\Resolver\LoadsSourceFromArgs;
use ZOOlanders\YOOessentials\Source\Provider\GoogleCalendar\GoogleCalendarSource;

class GoogleCalendarEventsQueryType extends AbstractQueryType implements HasSourceInterface
{
    use LoadsSourceFromArgs, CachesResolvedSourceData;

    public function name(): string
    {
        return "googleCalendarEvents_{$this->source()->id()}_query";
    }

    public function label(): string
    {
        return "{$this->source()->name()} Events";
    }

    public static function getCacheKey(): string
    {
        return 'google-calendar-events';
    }

    public function config(): array
    {
        return [
            'fields' => [
                $this->name() => [
                    'type' => [
                        'listOf' => GoogleCalendarEventType::TYPE_NAME,
                    ],

                    'args' => [
                        'q' => [
                            'type' => 'String',
                        ],
                        'orderBy' => [
                            'type' => 'String',
                        ],
                        'maxResults' => [
                            'type' => 'Int',
                        ],
                        'timeMin' => [
                            'type' => 'String',
                        ],
                        'timeMax' => [
                            'type' => 'String',
                        ],
                        'cache' => [
                            'type' => 'Int',
                        ],
                    ],

                    'metadata' => [
                        'group' => 'Google Calendar',
                        'label' => $this->label(),
                        'fields' => [
                            'q' => [
                                'label' => 'Query',
                                'description' => 'Filter events that match these terms in the fields summary, description, location, attendee\'s displayName and attendee\'s email.',
                                'source' => 'true',
                            ],
                            '_order_limit' => [
                                'type' => 'grid',
                                'width' => '1-2',
                                'description' => 'Set the order and limit the number of events.',
                                'fields' => [
                                    'orderBy' => [
                                        'label' => 'Order By',
                                        'type' => 'select',
                                        'default' => 'startTime',
                                        'options' => [
                                            'Start Time (asc)' => 'startTime',
                                            'Updated (asc)' => 'updated',
                                        ],
                                    ],
                                    'maxResults' => [
                                        'label' => 'Quantity',
                                        'type' => 'yooessentials-number',
                                        'default' => 250,
                                        'source' => 'true',
                                        'attrs' => [
                                            'min' => 1,
                                            'placeholder' => 250,
                                        ],
                                    ],
                                ],
                            ],
                            '_time_range' => [
                                'type' => 'grid',
                                'width' => '1-2',
                                'description' => 'Set the Lower and Upper bounds (exclusive) for an event\'s start and/or end time to filter by. Can be date without time, e.g. <code>2011-06-03</code> formated as <code>Y-m-d</code>, in which case the current time will be appended, or a timestamp with time zone offset, e.g. <code>2011-06-03T10:00:00-07:00</code> formated as <code>c</code>.',
                                'fields' => [
                                    'timeMin' => [
                                        'label' => 'Time Min',
                                        'type' => 'yooessentials-datetime',
                                        'small' => true,
                                        'source' => 'true',
                                    ],
                                    'timeMax' => [
                                        'label' => 'Time Max',
                                        'type' => 'yooessentials-datetime',
                                        'small' => true,
                                        'source' => 'true',
                                    ],
                                ],
                            ],
                            'cache' => $this->cacheField(),
                        ],
                    ],

                    'extensions' => [
                        'call' => [
                            'func' => __CLASS__ . '::resolve',
                            'args' => [
                                'source_id' => $this->source->id(),
                            ],
                        ],
                    ],
                ],
            ],
        ];
    }

    public static function resolve($root, array $args)
    {
        $source = self::loadSource($args, GoogleCalendarSource::class);

        if (!$source) {
            return [];
        }

        $args['singleEvents'] = true;

        return self::resolveFromCache($source, $args, function () use ($source, $args) {
            return (array) $source->api()->events($source->config('calendar'), $args);
        });
    }
}
