<?php

namespace ZOOlanders\YOOessentials\Vendor\AsyncAws\S3\Input;

use ZOOlanders\YOOessentials\Vendor\AsyncAws\Core\Exception\InvalidArgument;
use ZOOlanders\YOOessentials\Vendor\AsyncAws\Core\Input;
use ZOOlanders\YOOessentials\Vendor\AsyncAws\Core\Request;
use ZOOlanders\YOOessentials\Vendor\AsyncAws\Core\Stream\StreamFactory;
use ZOOlanders\YOOessentials\Vendor\AsyncAws\S3\Enum\ChecksumAlgorithm;
use ZOOlanders\YOOessentials\Vendor\AsyncAws\S3\Enum\RequestPayer;
final class UploadPartRequest extends Input
{
    /**
     * Object data.
     *
     * @var string|resource|callable|iterable|null
     */
    private $body;
    /**
     * The name of the bucket to which the multipart upload was initiated.
     *
     * @required
     *
     * @var string|null
     */
    private $bucket;
    /**
     * Size of the body in bytes. This parameter is useful when the size of the body cannot be determined automatically.
     *
     * @var string|null
     */
    private $contentLength;
    /**
     * The base64-encoded 128-bit MD5 digest of the part data. This parameter is auto-populated when using the command from
     * the CLI. This parameter is required if object lock parameters are specified.
     *
     * @var string|null
     */
    private $contentMd5;
    /**
     * Indicates the algorithm used to create the checksum for the object when using the SDK. This header will not provide
     * any additional functionality if not using the SDK. When sending this header, there must be a corresponding
     * `x-amz-checksum` or `x-amz-trailer` header sent. Otherwise, Amazon S3 fails the request with the HTTP status code
     * `400 Bad Request`. For more information, see Checking object integrity in the *Amazon S3 User Guide*.
     *
     * @see https://docs.aws.amazon.com/AmazonS3/latest/userguide/checking-object-integrity.html
     *
     * @var ChecksumAlgorithm::*|null
     */
    private $checksumAlgorithm;
    /**
     * This header can be used as a data integrity check to verify that the data received is the same data that was
     * originally sent. This header specifies the base64-encoded, 32-bit CRC32 checksum of the object. For more information,
     * see Checking object integrity in the *Amazon S3 User Guide*.
     *
     * @see https://docs.aws.amazon.com/AmazonS3/latest/userguide/checking-object-integrity.html
     *
     * @var string|null
     */
    private $checksumCrc32;
    /**
     * This header can be used as a data integrity check to verify that the data received is the same data that was
     * originally sent. This header specifies the base64-encoded, 32-bit CRC32C checksum of the object. For more
     * information, see Checking object integrity in the *Amazon S3 User Guide*.
     *
     * @see https://docs.aws.amazon.com/AmazonS3/latest/userguide/checking-object-integrity.html
     *
     * @var string|null
     */
    private $checksumCrc32C;
    /**
     * This header can be used as a data integrity check to verify that the data received is the same data that was
     * originally sent. This header specifies the base64-encoded, 160-bit SHA-1 digest of the object. For more information,
     * see Checking object integrity in the *Amazon S3 User Guide*.
     *
     * @see https://docs.aws.amazon.com/AmazonS3/latest/userguide/checking-object-integrity.html
     *
     * @var string|null
     */
    private $checksumSha1;
    /**
     * This header can be used as a data integrity check to verify that the data received is the same data that was
     * originally sent. This header specifies the base64-encoded, 256-bit SHA-256 digest of the object. For more
     * information, see Checking object integrity in the *Amazon S3 User Guide*.
     *
     * @see https://docs.aws.amazon.com/AmazonS3/latest/userguide/checking-object-integrity.html
     *
     * @var string|null
     */
    private $checksumSha256;
    /**
     * Object key for which the multipart upload was initiated.
     *
     * @required
     *
     * @var string|null
     */
    private $key;
    /**
     * Part number of part being uploaded. This is a positive integer between 1 and 10,000.
     *
     * @required
     *
     * @var int|null
     */
    private $partNumber;
    /**
     * Upload ID identifying the multipart upload whose part is being uploaded.
     *
     * @required
     *
     * @var string|null
     */
    private $uploadId;
    /**
     * Specifies the algorithm to use to when encrypting the object (for example, AES256).
     *
     * @var string|null
     */
    private $sseCustomerAlgorithm;
    /**
     * Specifies the customer-provided encryption key for Amazon S3 to use in encrypting data. This value is used to store
     * the object and then it is discarded; Amazon S3 does not store the encryption key. The key must be appropriate for use
     * with the algorithm specified in the `x-amz-server-side-encryption-customer-algorithm header`. This must be the same
     * encryption key specified in the initiate multipart upload request.
     *
     * @var string|null
     */
    private $sseCustomerKey;
    /**
     * Specifies the 128-bit MD5 digest of the encryption key according to RFC 1321. Amazon S3 uses this header for a
     * message integrity check to ensure that the encryption key was transmitted without error.
     *
     * @var string|null
     */
    private $sseCustomerKeyMd5;
    /**
     * @var RequestPayer::*|null
     */
    private $requestPayer;
    /**
     * The account ID of the expected bucket owner. If the bucket is owned by a different account, the request fails with
     * the HTTP status code `403 Forbidden` (access denied).
     *
     * @var string|null
     */
    private $expectedBucketOwner;
    /**
     * @param array{
     *   Body?: string|resource|callable|iterable,
     *   Bucket?: string,
     *   ContentLength?: string,
     *   ContentMD5?: string,
     *   ChecksumAlgorithm?: ChecksumAlgorithm::*,
     *   ChecksumCRC32?: string,
     *   ChecksumCRC32C?: string,
     *   ChecksumSHA1?: string,
     *   ChecksumSHA256?: string,
     *   Key?: string,
     *   PartNumber?: int,
     *   UploadId?: string,
     *   SSECustomerAlgorithm?: string,
     *   SSECustomerKey?: string,
     *   SSECustomerKeyMD5?: string,
     *   RequestPayer?: RequestPayer::*,
     *   ExpectedBucketOwner?: string,
     *   @region?: string,
     * } $input
     */
    public function __construct(array $input = [])
    {
        $this->body = $input['Body'] ?? null;
        $this->bucket = $input['Bucket'] ?? null;
        $this->contentLength = $input['ContentLength'] ?? null;
        $this->contentMd5 = $input['ContentMD5'] ?? null;
        $this->checksumAlgorithm = $input['ChecksumAlgorithm'] ?? null;
        $this->checksumCrc32 = $input['ChecksumCRC32'] ?? null;
        $this->checksumCrc32C = $input['ChecksumCRC32C'] ?? null;
        $this->checksumSha1 = $input['ChecksumSHA1'] ?? null;
        $this->checksumSha256 = $input['ChecksumSHA256'] ?? null;
        $this->key = $input['Key'] ?? null;
        $this->partNumber = $input['PartNumber'] ?? null;
        $this->uploadId = $input['UploadId'] ?? null;
        $this->sseCustomerAlgorithm = $input['SSECustomerAlgorithm'] ?? null;
        $this->sseCustomerKey = $input['SSECustomerKey'] ?? null;
        $this->sseCustomerKeyMd5 = $input['SSECustomerKeyMD5'] ?? null;
        $this->requestPayer = $input['RequestPayer'] ?? null;
        $this->expectedBucketOwner = $input['ExpectedBucketOwner'] ?? null;
        parent::__construct($input);
    }
    public static function create($input) : self
    {
        return $input instanceof self ? $input : new self($input);
    }
    /**
     * @return string|resource|callable|iterable|null
     */
    public function getBody()
    {
        return $this->body;
    }
    public function getBucket() : ?string
    {
        return $this->bucket;
    }
    /**
     * @return ChecksumAlgorithm::*|null
     */
    public function getChecksumAlgorithm() : ?string
    {
        return $this->checksumAlgorithm;
    }
    public function getChecksumCrc32() : ?string
    {
        return $this->checksumCrc32;
    }
    public function getChecksumCrc32C() : ?string
    {
        return $this->checksumCrc32C;
    }
    public function getChecksumSha1() : ?string
    {
        return $this->checksumSha1;
    }
    public function getChecksumSha256() : ?string
    {
        return $this->checksumSha256;
    }
    public function getContentLength() : ?string
    {
        return $this->contentLength;
    }
    public function getContentMd5() : ?string
    {
        return $this->contentMd5;
    }
    public function getExpectedBucketOwner() : ?string
    {
        return $this->expectedBucketOwner;
    }
    public function getKey() : ?string
    {
        return $this->key;
    }
    public function getPartNumber() : ?int
    {
        return $this->partNumber;
    }
    /**
     * @return RequestPayer::*|null
     */
    public function getRequestPayer() : ?string
    {
        return $this->requestPayer;
    }
    public function getSseCustomerAlgorithm() : ?string
    {
        return $this->sseCustomerAlgorithm;
    }
    public function getSseCustomerKey() : ?string
    {
        return $this->sseCustomerKey;
    }
    public function getSseCustomerKeyMd5() : ?string
    {
        return $this->sseCustomerKeyMd5;
    }
    public function getUploadId() : ?string
    {
        return $this->uploadId;
    }
    /**
     * @internal
     */
    public function request() : Request
    {
        // Prepare headers
        $headers = [];
        if (null !== $this->contentLength) {
            $headers['Content-Length'] = $this->contentLength;
        }
        if (null !== $this->contentMd5) {
            $headers['Content-MD5'] = $this->contentMd5;
        }
        if (null !== $this->checksumAlgorithm) {
            if (!ChecksumAlgorithm::exists($this->checksumAlgorithm)) {
                throw new InvalidArgument(\sprintf('Invalid parameter "ChecksumAlgorithm" for "%s". The value "%s" is not a valid "ChecksumAlgorithm".', __CLASS__, $this->checksumAlgorithm));
            }
            $headers['x-amz-sdk-checksum-algorithm'] = $this->checksumAlgorithm;
        }
        if (null !== $this->checksumCrc32) {
            $headers['x-amz-checksum-crc32'] = $this->checksumCrc32;
        }
        if (null !== $this->checksumCrc32C) {
            $headers['x-amz-checksum-crc32c'] = $this->checksumCrc32C;
        }
        if (null !== $this->checksumSha1) {
            $headers['x-amz-checksum-sha1'] = $this->checksumSha1;
        }
        if (null !== $this->checksumSha256) {
            $headers['x-amz-checksum-sha256'] = $this->checksumSha256;
        }
        if (null !== $this->sseCustomerAlgorithm) {
            $headers['x-amz-server-side-encryption-customer-algorithm'] = $this->sseCustomerAlgorithm;
        }
        if (null !== $this->sseCustomerKey) {
            $headers['x-amz-server-side-encryption-customer-key'] = $this->sseCustomerKey;
        }
        if (null !== $this->sseCustomerKeyMd5) {
            $headers['x-amz-server-side-encryption-customer-key-MD5'] = $this->sseCustomerKeyMd5;
        }
        if (null !== $this->requestPayer) {
            if (!RequestPayer::exists($this->requestPayer)) {
                throw new InvalidArgument(\sprintf('Invalid parameter "RequestPayer" for "%s". The value "%s" is not a valid "RequestPayer".', __CLASS__, $this->requestPayer));
            }
            $headers['x-amz-request-payer'] = $this->requestPayer;
        }
        if (null !== $this->expectedBucketOwner) {
            $headers['x-amz-expected-bucket-owner'] = $this->expectedBucketOwner;
        }
        // Prepare query
        $query = [];
        if (null === ($v = $this->partNumber)) {
            throw new InvalidArgument(\sprintf('Missing parameter "PartNumber" for "%s". The value cannot be null.', __CLASS__));
        }
        $query['partNumber'] = (string) $v;
        if (null === ($v = $this->uploadId)) {
            throw new InvalidArgument(\sprintf('Missing parameter "UploadId" for "%s". The value cannot be null.', __CLASS__));
        }
        $query['uploadId'] = $v;
        // Prepare URI
        $uri = [];
        if (null === ($v = $this->bucket)) {
            throw new InvalidArgument(\sprintf('Missing parameter "Bucket" for "%s". The value cannot be null.', __CLASS__));
        }
        $uri['Bucket'] = $v;
        if (null === ($v = $this->key)) {
            throw new InvalidArgument(\sprintf('Missing parameter "Key" for "%s". The value cannot be null.', __CLASS__));
        }
        $uri['Key'] = $v;
        $uriString = '/' . \rawurlencode($uri['Bucket']) . '/' . \str_replace('%2F', '/', \rawurlencode($uri['Key']));
        // Prepare Body
        $body = $this->body ?? '';
        // Return the Request
        return new Request('PUT', $uriString, $query, $headers, StreamFactory::create($body));
    }
    /**
     * @param string|resource|callable|iterable|null $value
     */
    public function setBody($value) : self
    {
        $this->body = $value;
        return $this;
    }
    public function setBucket(?string $value) : self
    {
        $this->bucket = $value;
        return $this;
    }
    /**
     * @param ChecksumAlgorithm::*|null $value
     */
    public function setChecksumAlgorithm(?string $value) : self
    {
        $this->checksumAlgorithm = $value;
        return $this;
    }
    public function setChecksumCrc32(?string $value) : self
    {
        $this->checksumCrc32 = $value;
        return $this;
    }
    public function setChecksumCrc32C(?string $value) : self
    {
        $this->checksumCrc32C = $value;
        return $this;
    }
    public function setChecksumSha1(?string $value) : self
    {
        $this->checksumSha1 = $value;
        return $this;
    }
    public function setChecksumSha256(?string $value) : self
    {
        $this->checksumSha256 = $value;
        return $this;
    }
    public function setContentLength(?string $value) : self
    {
        $this->contentLength = $value;
        return $this;
    }
    public function setContentMd5(?string $value) : self
    {
        $this->contentMd5 = $value;
        return $this;
    }
    public function setExpectedBucketOwner(?string $value) : self
    {
        $this->expectedBucketOwner = $value;
        return $this;
    }
    public function setKey(?string $value) : self
    {
        $this->key = $value;
        return $this;
    }
    public function setPartNumber(?int $value) : self
    {
        $this->partNumber = $value;
        return $this;
    }
    /**
     * @param RequestPayer::*|null $value
     */
    public function setRequestPayer(?string $value) : self
    {
        $this->requestPayer = $value;
        return $this;
    }
    public function setSseCustomerAlgorithm(?string $value) : self
    {
        $this->sseCustomerAlgorithm = $value;
        return $this;
    }
    public function setSseCustomerKey(?string $value) : self
    {
        $this->sseCustomerKey = $value;
        return $this;
    }
    public function setSseCustomerKeyMd5(?string $value) : self
    {
        $this->sseCustomerKeyMd5 = $value;
        return $this;
    }
    public function setUploadId(?string $value) : self
    {
        $this->uploadId = $value;
        return $this;
    }
}
